#include <string>
#include <list>

#include "script.h"
#include "script_manager.h"
#include "lunar.h"


ScriptManager::ScriptManager()
{
  this->init();
}

ScriptManager::~ScriptManager()
{


}


void ScriptManager::loadParams(const TiXmlElement* root)
{
  //BaseObject::loadParams(root);

  const TiXmlElement* scripts = root->FirstChildElement("Scripts");

  if( scripts == NULL)
  {
    PRINTF(1)("ScriptManager is missing 'Scripts'\n");
  }
  else
  {
    createScriptList(scripts);
  }


}

void ScriptManager::tick(float timestep)
{
}


void ScriptManager::initScripts()
{
  for(std::list<LuaScript>::iterator it = scriptList.begin(); it != scriptList.end(); it++ )
  {
    (*it).script.loadFile((*it).name);
  }
}

void ScriptManager::init()
{
  currentWorld.assign("");

}


void  ScriptManager::createScriptList(const TiXmlElement* scripts)
{
  const TiXmlElement* script = scripts->FirstChildElement();

  PRINTF(4)("Creating scriptlist\n");

  while( script != NULL)
  {
    { // scope to make the LoadParam work
    LoadParam(script, "file", this, ScriptManager, setCurrentScriptFile)
        .describe("the fileName of the script, that should be loaded into this world")
        .defaultValues("");
    }

    if(!currentScript.name.empty()) // if LoadParam didn't fail... fill the object list with the needed objects
    {

    LOAD_PARAM_START_CYCLE(script, object);
    {
     // LoadParam_CYCLE(object, "object", this, ScriptManager, addObjectToScript)
       //   .describe("The name of an object that is needed by a script");
    }
    LOAD_PARAM_END_CYCLE(object);

    scriptList.push_back(currentScript);
    }

    script = script->NextSiblingElement("Script");

  }


}


Script* ScriptManager::getScriptByFile(std::string& file)
{

  for(std::list<LuaScript>::iterator it = scriptList.begin(); it != scriptList.end(); it++ )
  {
    if( (*it).name.compare(file) == 0)
    {
      return &((*it).script);
    }
  }
  return NULL;

}

void  ScriptManager::setCurrentScriptFile(const std::string& file)
 {
   if( !fileIsInScriptList(file) )
     currentScript.name = file;
   else
     currentScript.name.assign("");
 }

void  ScriptManager::addObjectToScript(const std::string object)
{
  if(!objectIsInObjectList(object,currentScript))
    currentScript.objectList.push_back(object);
}


bool ScriptManager::fileIsInScriptList(const std::string& file)
{
  for(std::list<LuaScript>::const_iterator it = scriptList.begin(); it != scriptList.end(); it++ )
  {
    if( (*it).name.compare(file) == 0)
    {
      return true;
    }
  }
  return false;
}


bool ScriptManager::objectIsInObjectList(const std::string& object, const LuaScript& script)
{
  for(std::list<std::string>::const_iterator it = script.objectList.begin(); it != script.objectList.end(); it++ )
  {
    if( (*it).compare(object) == 0)
    {
      return true;
    }
  }
  return false;
}


